extends CharacterBody2D

@onready var starting_moving_speed = movement_speed
enum state {
	RUNNING,
	DYING,
	ATTACKING
}
var current_state = 0
var current_target
var hp = 3
var movement_speed = 15

var enemies = {
	'orc':
		{
			'hp': 3,
			'movement_speed': 15,
			'damage': 1,
			'running_texture': load("res://Sprites/Enemy/Orc Crew/Orc/Run/Run-Sheet.png"),
			'dying_texture': load("res://Sprites/Enemy/Orc Crew/Orc/Death/Death-Sheet.png"),
			'attacking_texture': load("res://Sprites/Enemy/Orc Crew/Orc/Idle/Idle-Sheet.png")
		},
	'orc-rogue':
		{
			'hp': 3,
			'movement_speed': 21,
			'damage': 1,
			'running_texture': load("res://Sprites/Enemy/Orc Crew/Orc - Rogue/Run/Run-Sheet.png"),
			'dying_texture': load("res://Sprites/Enemy/Orc Crew/Orc - Rogue/Death/Death-Sheet.png"),
			'attacking_texture': load("res://Sprites/Enemy/Orc Crew/Orc - Rogue/Idle/Idle-Sheet.png")
		},
	'orc-shaman':
		{
			'hp': 3,
			'movement_speed': 5,
			'damage': 1,
			'running_texture': load("res://Sprites/Enemy/Orc Crew/Orc - Shaman/Run/Run-Sheet.png"),
			'dying_texture': load("res://Sprites/Enemy/Orc Crew/Orc - Shaman/Death/Death-Sheet.png"),
			'attacking_texture': load("res://Sprites/Enemy/Orc Crew/Orc - Shaman/Idle/Idle-Sheet.png")
		},
	'orc-warrior':
		{
			'hp': 10,
			'movement_speed': 10,
			'damage': 2,
			'running_texture': load("res://Sprites/Enemy/Orc Crew/Orc - Warrior/Run/Run-Sheet.png"),
			'dying_texture': load("res://Sprites/Enemy/Orc Crew/Orc - Warrior/Death/Death-Sheet.png"),
			'attacking_texture': load("res://Sprites/Enemy/Orc Crew/Orc - Warrior/Idle/Idle-Sheet.png")
		}
}

var enemy_type

func _ready() -> void:
	# pick a random between orc and orc-rogue
	set_enemy_type(['orc', 'orc-rogue', 'orc-shaman', 'orc-warrior'].pick_random())
	
	trigger_state(state.RUNNING)
	$AnimationPlayer.animation_finished.connect(animation_ended)


func set_enemy_type(type):
	enemy_type = type
	hp = enemies[enemy_type]['hp']
	movement_speed = enemies[enemy_type]['movement_speed']
	starting_moving_speed = movement_speed
	$Sprite2D.texture = enemies[enemy_type]['running_texture']


func trigger_state(c_state):
	current_state = c_state
	if c_state == state.RUNNING:
		$Sprite2D.texture = enemies[enemy_type]['running_texture']
		$Sprite2D.hframes = 6
		$Sprite2D.offset.y = 0
		$AnimationPlayer.play("running")
		$AnimationPlayer.speed_scale = movement_speed * 0.25
	if c_state == state.DYING:
		$Sprite2D.texture = enemies[enemy_type]['dying_texture']
		$Sprite2D.hframes = 6
		$Sprite2D.offset.y = 0
		$AnimationPlayer.play("running") #both share the same amount of frames
		$AnimationPlayer.speed_scale = 3
		movement_speed = 0
	if c_state == state.ATTACKING:
		$Sprite2D.texture = enemies[enemy_type]['attacking_texture']
		$Sprite2D.hframes = 4
		$Sprite2D.offset.y = 17
		$AnimationPlayer.play("idle") #both share the same amount of frames
		$AnimationPlayer.speed_scale = 0.5
		movement_speed = 0


func _process(delta: float) -> void:
	var collision = move_and_collide(Vector2(-movement_speed * delta, 0))
	if collision:
		if collision.get_collider().is_in_group("soldier"):
			if current_state != state.ATTACKING:
				current_target = collision.get_collider().get_parent()
				trigger_state(state.ATTACKING)			
				print("I collided with ", collision.get_collider().name)
	
	if hp <= 0:
		if current_state != state.DYING:
			trigger_state(state.DYING)


func animation_ended(_anim_name) -> void:
	if current_state == state.RUNNING:
		$AnimationPlayer.play("running")
	elif current_state == state.DYING:
		Game.spawn_gold(global_position)
		queue_free()
	elif current_state == state.ATTACKING:
		# await get_tree().create_timer(1.0).timeout # This doesn't work
		if current_target != null:
			current_target.hp -= enemies[enemy_type]['damage']
			print("I attacked ", current_target.name)
			if current_target.hp <= 0:
				trigger_state(state.RUNNING)
			else:
				trigger_state(state.ATTACKING)
		movement_speed = starting_moving_speed
